﻿namespace RenderTargetBitmapDemo
{
    using System.IO;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;
    using System.Windows.Media.Imaging;
    using Microsoft.Win32;

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            this.InitializeComponent();
        }

        /// <summary>Renders the Visual object and store it to file.</summary>
        /// <param name="baseElement">The Visual object to be used as a bitmap. </param>
        /// <param name="imageWidth">The height of the bitmap.</param>
        /// <param name="imageHeight">The width of the bitmap.</param>
        /// <param name="pathToOutputFile">Full path to the output file.</param>
        private void SaveControlImage(Visual baseElement, int imageWidth, int imageHeight, string pathToOutputFile)
        {
            // 1) get current dpi
            PresentationSource pSource = PresentationSource.FromVisual(Application.Current.MainWindow);
            Matrix m = pSource.CompositionTarget.TransformToDevice;
            double dpiX = m.M11 * 96;
            double dpiY = m.M22 * 96;

            // 2) create RenderTargetBitmap
            var elementBitmap = new RenderTargetBitmap(imageWidth, imageHeight, dpiX, dpiY, PixelFormats.Default);

            // 3) undo element transformation
            var drawingVisual = new DrawingVisual();
            using (DrawingContext drawingContext = drawingVisual.RenderOpen()) {
                var visualBrush = new VisualBrush(baseElement);
                drawingContext.DrawRectangle(
                    visualBrush,
                    null,
                    new Rect(new Point(0, 0), new Size(imageWidth / m.M11, imageHeight / m.M22)));
            }

            // 4) draw element
            elementBitmap.Render(drawingVisual);

            // 5) create PNG image
            var encoder = new PngBitmapEncoder();
            encoder.Frames.Add(BitmapFrame.Create(elementBitmap));

            // 6) save image to file
            using (var imageFile = new FileStream(pathToOutputFile, FileMode.Create, FileAccess.Write)) {
                encoder.Save(imageFile);
                imageFile.Flush();
                imageFile.Close();
            }
        }

        /// <summary>Shows dialog that allows the user to specify a filename to save a file as. </summary>
        /// <returns>Returns a string containing the full path of the file selected or string.Empty.</returns>
        private string GetOutputFileName()
        {
            var dlgSave = new SaveFileDialog {
                DefaultExt = "png",
                Title = "Choose output file",
                Filter = "PNG Files|*.png"
            };

            if (dlgSave.ShowDialog() == true)
                return dlgSave.FileName;

            return string.Empty;
        }

        /// <summary>Button "Save to bitmap" click handler.</summary>
        private void _btnSaveToBitmap_Click(object sender, RoutedEventArgs e)
        {
            // get output file name
            string fileName = this.GetOutputFileName();
            if (fileName == string.Empty)
                return;

            // select visual element
            Visual element = null;
            var elementWidth = 0;
            var elementHeight = 0;

            if (this._rbSaveGrid.IsChecked == true) {
                element = this._baseGrid;
                elementWidth = (int)this._baseGrid.ActualWidth;
                elementHeight = (int)this._baseGrid.ActualHeight;
            }
            else {
                element = this._tabControl1;
                elementWidth = (int)this._tabControl1.ActualWidth;
                elementHeight = (int)this._tabControl1.ActualHeight;
            }

            // save selected element to file
            this.SaveControlImage(element, elementWidth, elementHeight, fileName);
        }

        /// <summary>Button "Print" click handler.</summary>
        private void _btnPrint_Click(object sender, RoutedEventArgs e)
        {
            var dlgPrint = new PrintDialog();

            // select visual element
            Visual element = null;
            if (this._rbSaveGrid.IsChecked == true) {
                element = this._baseGrid;
            }
            else {
                element = this._tabControl1;
            }

            // Print
            if (dlgPrint.ShowDialog() == true) {
                dlgPrint.PrintVisual(element, "The picture is drawn dynamically");
            }
        }
    }
}